declare module "Emitter" {
    export class Emitter {
        private eventCallbacks;
        /**
         * Register an event type by name. This method is called when the 'on' method is called if an event of that type has
         * not been registered yet.
         * @param eventName
         */
        register(eventName: string): void;
        /**
         * Add an event callback with optional context
         * @param eventName
         * @param callback
         * @param context
         */
        on(eventName: string, callback: Function, context?: object): void;
        /**
         * Remove an event callback by reference
         * @param eventName
         * @param [callback]
         * @param [force=false]
         */
        off(eventName: string): void;
        off(eventName: string, callback: Function): void;
        off(eventName: string, callback: Function, force: boolean): void;
        /**
         * Dispatch an event with eventName using the 'call' method
         * @param eventName
         * @param args
         */
        dispatch(eventName: string, ...args: any[]): void;
        /**
         * Dispatch an event with eventName using the 'apply' method
         * @param eventName
         * @param args
         */
        dispatchApply(eventName: string, args?: any[]): void;
        /**
         * Alias for the dispatch method
         * @param eventName
         * @param args
         */
        emit(eventName: string, ...args: any[]): void;
        /**
         * Remove all listeners from this instance
         */
        removeListeners(): void;
        /**
         * Getter to check if any listeners are present on this emitter
         */
        hasListeners(): boolean;
    }
}
declare module "Math" {
    /**
     * Set the seed for our prng and switch to a simple, seedable prng
     *
     * ```typescript
     * import { randomInt, setSeed } from 'goodish'
     * setSeed(11)
     * randomInt(1, 100) // always 44
     * ```
     *
     * @param seed
     */
    export function setSeed(seed: number): void;
    /**
     * Remove any previously set seed on the random number generator
     *
     * ```typescript
     * import { clearSeed } from 'goodish'
     * clearSeed() // Now randomized functions will use Math.random again
     * ```
     */
    export function clearSeed(): void;
    /**
     * Returns a random float between min and max.
     *
     * ```typescript
     * import { random } from 'goodish'
     * console.log(random()) // value between 0 and 1
     * console.log(random(0, 100) // value between 0 and 100
     *
     * @param min
     * @param max
     */
    export function random(min?: number, max?: number): number;
    /**
     * Returns a random integer between the min and max values.
     *
     * ```typescript
     * import { randomInt } from 'goodish'
     * console.log(randomInt(0, 100)) // Some integer between 0 and 100
     * ```
     *
     * @param min
     * @param max
     */
    export function randomInt(min: number, max: number): number;
    /**
     * Round a number to the specified number of digits
     *
     * ```typescript
     * import { toFixedNum } from 'goodish'
     * console.log(toFixedNum(3.14159, 2)) // 3.14
     * ```
     *
     * @param float
     * @param digits
     */
    export function toFixedNum(float: number, digits: number): number;
    /**
     * Calculate the greatest common divisor of two numbers
     *
     * ```typescript
     * import { greatestCommonDivisor } from 'goodish'
     * console.log(greatestCommonDivisor(18, 12)) // 6
     * ```
     *
     * @param numA
     * @param numB
     * @reference Adapted from [this site](https://www.w3resource.com/javascript-exercises/javascript-math-exercise-8.php)
     */
    export function greatestCommonDivisor(numA: number, numB: number): number;
    /**
     * Calculate the lowest common multiple of two numbers
     *
     * ```typescript
     * import { lowestCommonMultiple } from 'goodish'
     * console.log(lowestCommonMultiple(6, 8)) // 24
     * ```
     *
     * @param numA
     * @param numB
     * @reference Adapted from [this site](https://www.w3resource.com/javascript-exercises/javascript-math-exercise-10.php)
     */
    export function lowestCommonMultiple(numA: number, numB: number): number;
    /**
     * Clamp a value between the min and max values. If it is below min or above max then we set the value equal to the
     * respective boundary.
     *
     * ```typescript
     * import { clamp } from 'goodish'
     * console.log(clamp(100, 0, 10)) // 10
     * ```
     *
     * @param val
     * @param min
     * @param max
     */
    export function clamp(val: number, min: number, max: number): number;
    /**
     * Wrap the value within these bounds. More explanation of the idea (here)[https://en.wikipedia.org/wiki/Wrapping_(graphics)]
     *
     * ```typescript
     * import { wrap } from 'goodish'
     * console.log(wrap(5, 1, 4)) // 2 because value exceeds the max val by 1 and wraps around from the lower boundary
     * ```
     *
     * @param val
     * @param min
     * @param max
     */
    export function wrap(val: number, min: number, max: number): number;
}
declare module "Iter" {
    /**
     * Returns an array of values between the min and max values.
     *
     * ```typescript
     * import { rangeArr } from 'goodish'
     * const arr = rangeArr(0, 5) // [0, 1, 2, 3, 4, 5]
     * ```
     *
     * @param min
     * @param max
     * @param step
     */
    export function rangeArr(min: number, max: number, step?: number): number[];
    /**
     * A generator which yields each value between the min and max values
     *
     * ```typescript
     * import { range } from 'goodish'
     * for (const n of range(0, 5)) {
     *   console.log(n) // 0, 1, 2, 3, 4
     * }
     * ```
     * @param min
     * @param max
     * @param step
     */
    export function range(min: number, max: number, step?: number): IterableIterator<number>;
    /**
     * Count the number of items in an iterator
     *
     * ```typescript
     * import { count } from 'goodish'
     * console.log(count([0, 1, 2, 3, 4])) // 5
     * ```
     *
     * @param arr
     */
    export function count(arr: Iterable<any>): number;
    /**
     * Returns a tuple containing a random element from a collection and the index of that element in the collection.
     *
     * ```typescript
     * import { randomFrom, setSeed } from 'goodish'
     * const arr = [0, 1, 2, 3, 4]
     * console.log(randomFrom(arr)) // any single element of arr. will be different each time this code runs
     * setSeed(1)
     * console.log(randomFrom(arr)) // always the same element of arr
     * ```
     * @param arr
     */
    export function randomFrom<T>(arr: T[]): [T, number];
    /**
     * Returns an array of N randomly selected elements from the supplied array
     * ```typescript
     * import { randomNFrom, setSeed } from 'goodish'
     * const arr = 'abcdefghijklmnopqrstuvwxyz0123456789'
     * console.log(randomNFrom(arr, 10)) // a random alphanumeric array with 10 characters
     * setSeed(1)
     * console.log(randomNFrom(arr, 10).join('')) // always the same alphanumeric string
     * ```
     * @param arr
     * @param n
     */
    export function randomNFrom<T>(arr: T[], n: number): T[];
    /**
     * Shuffle an array in place.
     *
     * ```typescript
     * import { shuffle, setSeed } from 'goodish'
     * const arr = [0, 1, 2, 3, 4]
     * console.log(shuffle(arr.slice())) // a different array order each time this code is run
     * setSeed(1)
     * console.log(shuffle(arr.slice()) // a different array order, but will be the same each time the code is run
     * ```
     *
     * @param arr
     * @reference Adapted from [here](https://stackoverflow.com/a/6274381/5551941)
     */
    export function shuffle<T>(arr: T[]): T[];
    /**
     * A generator which yields all of the permutations of an array
     *
     * ```typescript
     * import { permutationsOf } from 'goodish'
     * console.log(Array.from(permutationsOf([0, 1, 2, 3, 4]))
     * ```
     *
     * @param arr
     * @reference Adapted from [this answer](https://stackoverflow.com/a/37580979/5551941)
     */
    export function permutationsOf<T>(arr: T[]): IterableIterator<T[]>;
    /**
     * A generator which yields all of the combinations of N items from an array.
     *
     * ```typescript
     * import { combinationsOf } from 'goodish'
     * console.log(Array.from(combinationsOf([0, 1, 2], 2)) // [[0, 1], [0, 2], [1, 2]]
     * ```
     *
     * @param arr
     * @param n
     * @reference Copied verbatim from [this site](https://lowrey.me/es6-javascript-combination-generator/)
     */
    export function combinationsOf<T>(arr: T[], n: number): IterableIterator<T[]>;
    /**
     * Swap two indexes of an array
     *
     * ```typescript
     * import { swap } from 'goodish'
     * const arr = [0, 1, 2]
     * swap(1, 2)
     * console.log(arr) // [0, 2, 1]
     * ```
     *
     * @param arr
     * @param indexA
     * @param indexB
     */
    export function swap<T>(arr: T[], indexA: number, indexB: number): T[];
    /**
     * Transpose a 2D array.
     *
     * ```typescript
     * import { transpose } from 'goodish'
     * console.log(transpose([[0], [1], [2]])) // [[0, 1, 2]]
     * ```
     *
     * @param arr
     * @reference Copied verbatim from [this answer](https://stackoverflow.com/a/17428705/5551941)
     */
    export function transpose<T>(arr: T[][]): T[][];
}
declare module "Point2D" {
    export interface Point2D {
        x: number;
        y: number;
    }
}
declare module "Rectangle" {
    import { Point2D } from "Point2D";
    export class Rectangle {
        x: number;
        y: number;
        right: number;
        bottom: number;
        constructor(x: number, y: number, width: number, height: number);
        left: number;
        top: number;
        height: number;
        width: number;
        /**
         * Check if this rectangle intersects another rectangle
         * @param rect
         */
        intersects(rect: Rectangle): boolean;
        /**
         * Check if this rectangle contains a point.
         * @param point
         */
        contains(point: Point2D): boolean;
    }
}
declare module "Timing" {
    /**
     * An asynchronous wait function. Just useful for throttling purposes.
     * @param ms  - The number of milliseconds to wait before resolving the promise
     */
    export function wait(ms: number): Promise<void>;
    /**
     * A shared throttle. Safe to be used by multiple asynchronous requests at the same time.
     * @param freq - The number of milliseconds between resolution of this function
     */
    export function throttler(freq: number): Function;
}
declare module "Vector2D" {
    import { Point2D } from "Point2D";
    export class Vector2D {
        x: number;
        y: number;
        constructor(x: number, y: number);
        add(v: Point2D): this;
        subtract(v: Point2D): this;
        subtractC(v: Point2D): Vector2D;
        addC(v: Point2D): Vector2D;
        magSqrd(): number;
        mag(): number;
        dot(v: Point2D): number;
        normalize(mag?: number): this;
        normalizeC(mag?: number): Vector2D;
        zero(): this;
        rotate(angle: number): this;
        angle(): number;
        perpendicular(): Vector2D;
        angleBetween(v: Point2D): number;
        copy(): Vector2D;
        static midPoint(a: Point2D, b: Point2D): Vector2D;
        static fromPoints(a: Point2D, b: Point2D): Vector2D;
        static fromPoint(a: Point2D): Vector2D;
    }
}
declare module "index" {
    export * from "Iter";
    export * from "Math";
    export * from "Rectangle";
    export * from "Point2D";
    export * from "Emitter";
    export * from "Timing";
}
